function [beta, path, lambda] = NLARS(XtX, Xty, inc_path,maxK)

% Usage:
%   [beta, path, lambda] = NLARS(XtX, Xty, inc_path)
%
%   solves the problem argmin_beta 1/2||y-X*beta||_2^2 s.t. beta>=0
%   The problem is solved using a modification of the Least Angle Regression and Selection
%   algorithm. As such the entire regularization path for the LASSO problem
%   min 1/2||y-X*beta||_2^2 + lambda|beta|_1    s.t. beta>=0
%   for all values of lambda is given in path.
%
%   Pleace make reference to the following article describing the algorithm
%
%       Mrup, M and Madsen, K.H. and Hansen, L.K. "Approximate L0
%       constrained Non-negative Matrix and Tensor Factorization"   IEEE International Symposium on Circuits and Systems, ISCAS 2008.
%       pp. 1328 - 1331, 2008
%
%
% Input:
%       XtX         matrix of size D x D           
%       Xty         vector of size D x 1
%       inc_path    include the path in the output (default: false)
%       maxK        maximal number of active elements in beta (default: inf)
% Output:
%       beta    D x 1 solution vector 
%       path    the entire solution path
%       lambda  regularization value at beginning of step corresponds to
%               value of negative gradient
%
% Written by: Morten Mrup and Kristoffer Hougaard Madsen

if nargin<4
    maxK=inf;
end
if nargin<3
    inc_path=false;
end

p=length(Xty);
k=0;
if inc_path
    path=zeros(p,4*p);
    lambda=zeros(1,4*p);        
else 
    path=[];
    lambda=[];
end

% Initialization
I = 1:p; % inactive set
A = []; % active set
beta=zeros(size(Xty));

XtXbeta=XtX*beta;
c = (Xty - XtXbeta);
[C j] = max(c(I));
A = [A I(j)];
I(j) = [];

% insert initial zero solution in path.
k=k+1;
if inc_path    
    path(:,k)=beta;
    if isempty(C)
        lambda(k)=0;
    else
        lambda(k)=C;
    end
end
  

%% NLARS main loop
while sum(c(A))/length(A)>=1e-9 && nnz(beta)<maxK
    % if a variable has been dropped, continue iteration from zero without
    % introducing new variables, else introduce new variable to active
    % set        
    s = ones(length(A),1);
    w = (XtX(A,A))\s;       
    XtXw=XtX(:,A)*w;
    % Gradient in inactive set of same value as of active set
    gamma1=(C-c(I))./(XtXw(A(1))-XtXw(I));
    % Check step length towards zero for active elements
    gamma2=-beta(A)./w;
    % Check step to zero gradient for active elements
     gamma3=c(A(1))./XtXw(A(1));
    % Concatenate the three step types
    gamma=[gamma1; gamma2; gamma3];
    % Make negative and machine impression steps irrelevant
    gamma(gamma<=1e-9)=inf;
    % find smallest step
    [mu,t]=min(gamma);        
    % Update beta                       
    beta(A) = beta(A) + mu*w;

    % Check weather an active element has reached zero
    if t>length(gamma1) && t<length(gamma1)+length(gamma2)+1
        lassocond=1;
        j=t-length(gamma1);
        I = [I A(j)];
        A(j) = [];  
    else 
        lassocond=0;
    end
      
    % Recalculate gradient
    XtXbeta=XtX*beta;
    c = (Xty - XtXbeta);
    [C j] = max(c(I));

    % update path
    k=k+1;
    if inc_path        
        path(:,k)=beta;
        if isempty(C)
            lambda(k)=0;
        else
            lambda(k)=C;
        end
    end
    
    if ~lassocond             
        A = [A I(j)];
        I(j) = [];
    end        
end

% Trim path
if inc_path
    path=path(:,1:k);
    lambda=lambda(1:k);
end

